function w = kaiser(n_est,beta)
%KAISER Kaiser window.
%   W = KAISER(N,BETA) returns the BETA-valued N-point Kaiser window.
%
%   See also BARTLETT, BARTHANNWIN, BLACKMAN, BLACKMANHARRIS, BOHMANWIN, 
%            CHEBWIN, GAUSSWIN, HAMMING, HANN, NUTTALLWIN, RECTWIN, TRIANG, 
%            TUKEYWIN, WINDOW.

%   Author(s): L. Shure, 3-4-87
%   Copyright 1988-2002 The MathWorks, Inc.
%   $Revision: 1.15 $  $Date: 2002/03/28 17:28:33 $

error(nargchk(2,2,nargin));

[nn,w,trivialwin] = check_order(n_est);
if trivialwin, return, end;

nw = round(nn);
bes = abs(besseli(0,beta));
odd = rem(nw,2);
xind = (nw-1)^2;
n = fix((nw+1)/2);
xi = (0:n-1) + .5*(1-odd);
xi = 4*xi.^2;
w = besseli(0,beta*sqrt(1-xi/xind))/bes;
w = abs([w(n:-1:odd+1) w])';

    
% [EOF] kaiser.m
