function [s]= brown_noise(fs,sigsamples)
% % Title: Noise
% % Subtitle: Brown noise
% % Five 0.8 s samples of Brown Gaussian noise, called
% % Brown since it simulates Brownian motion. The power
% % spectrum of Brown noise is proportional to f^(-2)
% % (f is frequency), which means that it decreases with
% % 6 dB per octave.
% %
% % Note the random-walk-like structure in the signals!
% %
% % The samples are calculated by integrating white
% % noise. In order to prevent low-frequency random drifts
% % of the signal causing clipping of the 16-bit sound signals,
% % the noise is high-pass filtered with a 2nd-order Butterworth
% % filter.


dt = 1/fs;

% Calculate the coefficients of a 2nd-order 20-Hz
% high-pass Butterworth filter.
[b, a] = butter(2, 20/fs, 'high');

% Calculate 5 realisations of brown noise by
% integrating 5 realisations of white Gaussian noise.
% Then remove the inaudible low-frequency components.
%for i=1:5,
        % Calculate a realisation of white noise
        % with mean 0 and 1 standard deviation (sd).
        x = random('norm', 0, 1, 1, sigsamples);

        % Integrate the white noise and filter with the
        % 20-Hz high-pass filter.
        x = dt*cumsum(x);
        s = 600*filter(b, a, x);
        %end
% The next two operations are carried out
% in order to prevent clicks at the onset
% and offset of the sound.
s = FadeIn(s, fs, 0.02);
s = FadeOut(s, fs, 0.02);
%soundsc ([s], 44100)


%PlaySounds(s, sampleFrequency, interOnsetTime-duration)
%SaveSounds(s, sampleFrequency, interOnsetTime-duration,
%sprintf('..\\wavFiles\\%s.wav', mfilename));
%PlotSounds(s, sampleFrequency, 0.04, 0.05)
