function [s]= pink_noise(fs,samples)
% % Title: Noise
% % Subtitle: Pink noise
% % Five 0.8 s samples of pink noise are presented.
% % The power spectrum of pink noise is
% % proportional to f^(-1) (f is frequency), which
% % means that it decreases 3 dB per octave.
% % This means that frequency bands with the same
% % bandwidth on a logarithmic frequency scale
% % contain the same power.
% %
% % This pink noise was calculated according to
% % an algorithm I found on the internet
% % but forgot to note the name of the site and
% % could not refind it. The original author
% % mentions, however:
% % "Mr. Tom Bruhns from HP gave me the following pole/zero
% % locations for a white-to-pink noise filter for audio signals."
% % poles=[.9986823 .9914651 .9580812 .8090598 .2896591]';
% % zeros=[.9963594 .9808756 .9097290 .6128445 -.0324723]';
% %

poles=[.9986823 .9914651 .9580812 .8090598 .2896591]';
zeros=[.9963594 .9808756 .9097290 .6128445 -.0324723]';


% Find forward and backward coefficients of the filter
[b,a]=zp2tf(zeros,poles,1);

% Calculate 5 realisations of pink noise by
% filtering 5 realisations of white noise.
%for i=1:5,
        % Calculate a realisation of white noise
        % with mean 0 and 0.005 standard deviation (sd).
        % This sd is chosen so low, in order to
        % guarantee that the 'random walk' does not result
        % clipping. Compare with the sd's of 0.1 for
        % white noise and for pink noise.
        x = random('norm', 0, 0.1, 1, samples);

        % Filter the white noise to get pink noise.
        s = filter(b, a, x);
        
       
        % The next two operations are carried out
        % in order to prevent clicks at the onset
        % and offset of the sound.
        s = FadeIn(s, fs, 0.02);
        s = FadeOut(s, fs, 0.02);
        %end
       
