function oreja(action)
%OREJA software created for the design of psychoacoutic experiments.
% Elvira & Raul, November 2003

if nargin < 1
    action='init';
end

switch action
    case 'bugReport'
         web mailto:raul@ee.columbia.edu
    case 'help'
        open('manual.htm');
    case 'aboutBox'
        f=findobj('Tag','aboutWin');
        if ~isempty(f)
            figure(f);
        else  	
            pos=[100 100 573 419];
            fontsize = 10;
            aboutfig = figure('Units','pixels',...
                'Position',pos,...
                'Color',[1 1 1],...
                'Menubar','none',...   
                'Name','about...', ...
                'Tag','aboutWin', ...
                'NumberTitle','off');
            pheight=0.2;
            ax=axes('Parent',aboutfig,...
                'Color', 'none',...
                'XTick',[],...
                'YTick',[],...
                'Box','on'); 
            im=imread('orejalogo.png');
            image(im);
            set(ax,'XTick',[]);
            set(ax,'YTick',[]);
            set(ax,'Color','none');
            
        end
        
    case 'init'
        % does main figure window exist? If so, bring it to front

        
        f=findobj('Tag','oreja_fig');
        if ~isempty(f)
            figure(f);
        else
            orejaroot=cd;
            addpath(cd);
            addpath(strcat(orejaroot,'\library'));
            addpath(strcat(orejaroot,'\noises'));
            addpath(strcat(orejaroot,'\help_files'));
            savedir=pwd;  
            oreja_gui;		         % otherwise, create one
            ud=get(gcf,'UserData');
            ud.fs=8192;             
            ud.annotdir=strcat(orejaroot,'\data\annots\');
            ud.signaldir=strcat(orejaroot,'\data\sounds\');
            ud.savedir=ud.signaldir;
            ud.path=orejaroot;
            colormap(gray.*pink.*summer);
            ud.concat=0;
            ud.changefilter=0;
            ud.numchans = 10;
            ud.lines=[];
            ud.justplayed=0;
            ud.filename=[];
            ud.lowcf =100;
            ud.highcf=4000;
            ud.selected=zeros(1,ud.numchans);
            ud.channeliminated=0;
            ud.savename=[];
            ud.currentLabel=[];
            ud.dontask=0;
            ud.lastfullpath='';
            set([ud.chansmenu],'Enable','on');
            set(gcf,'UserData',ud);
            set(ud.chansmenu,'String',ud.numchans);
            ud.sig=[];
            ud.samps=length(ud.sig);
            set(ud.signalLine,'XData',1:ud.samps,'YData',ud.sig);

%             flast = fopen(strcat(orejaroot,'\oreja.ini'),'r');
%             counter = 0;
%             if flast ~= -1
%                 while ~feof(flast)
%                     counter = counter + 1;
%                     filelast=fgets(flast);
%                     if filelast ~= -1
%                         start = max(1,length(filelast)-18);
%                         shortfilename = strcat('...',filelast(start:length(filelast)));
%                         if counter == 1
%                             uimenu('Parent', findobj(gcf,'Tag','signalmenu'), 'Label', '---------------');
%                         end
%                         uimenu('Parent', findobj(gcf,'Tag','signalmenu'), 'Callback','oreja loadlast', 'Label', shortfilename, 'Tag', filelast);
%                     end
%                 end
%                 fclose(flast);
%             end

        end
                
        
    case 'createsig_oreja'
        ud=get(gcf,'UserData');
        channel_vect=ud.selected;
        noise=rand(size(ud.sig));
        %[sig,fs,name]=
        createsig_oreja(channel_vect,noise);
        
    case 'loadconcatenation'
        ud=get(gcf,'UserData');
        ud.concat=1;
        set(gcf,'UserData',ud);
        oreja 'load';
            
    case 'ERB'
        ud=get(gcf,'UserData');
        ud.changefilter=1;
        set(gcf,'UserData',ud);
        oreja 'load';
        
    case 'loadlast'
        lastfullpath = get(gcbo, 'Tag');
        ud=get(gcf,'UserData');
        ud.lastfullpath = lastfullpath;
        ud.dontask = 1;
        set(gcf,'UserData',ud);
        oreja 'load';
        
        
    case 'load'
        ud=get(gcf,'UserData');
        
        
        if ud.changefilter==1
            sig=ud.sig';
            name=ud.name;
            fs=ud.fs;

        else
            [name,sig,fs,fullpathname,savedir]=loadsig2(ud.savedir,ud.dontask,ud.lastfullpath);
            if savedir ~= 0
                ud.savedir = savedir;
            end
            ud.dontask=0;
        end
        if length(sig)
            if and(ud.concat == 1,isfield(ud,'sig'))
                ud.sig=[ud.sig sig']';
                sig=ud.sig;
                ud.concat=0;
            else
                ud.sig=sig;
            end
            
            delete(get(ud.wavAxes,'Children'));
            delete(get(ud.chanAxes,'Children'));
            delete(get(ud.signalAxes,'Children'));
            delete(get(ud.annotAxes,'Children'));
            
            
            ud.signalLine = line('Parent',ud.signalAxes, ...
                'ButtonDownFcn','oreja play', ...
                'Color',[0.7 0.2 1],...
                'XData',[0 0],'YData',[0 0]);

            
            
            ud.name=name; ud.fs=fs;
            ud.selected=ones(1,length(ud.selected));
            set(gcf,'UserData',ud);
            oreja 'newsig';
            ud=get(gcf,'UserData');
            set(ud.wavAxes,'XLim',[ud.wavAxesleft ud.wavAxesright]); 
            
            ud.left2 = round(0.1*(ud.wavAxesright-1))+1;
            ud.right2 = round(0.9*(ud.wavAxesright-1))+1;
            sig2=ud.sig(1:10:length(ud.sig))*100;
            ud.samps=length(sig);
            set(ud.signalAxes,'XLim',[1 length(sig)]);
            ud.annotAxesright=length(sig);
            ud.signalAxesright=0;
            ud.annotAxesleft=1;
            ud.left4 = round(0.1*(ud.annotAxesright-1))+1;    % cursors
            ud.right4 = round(0.9*(ud.annotAxesright-1))+1;
            set(ud.annotAxes,'Xlim',[ud.annotAxesleft ud.annotAxesright]);
            
            xlims=get(ud.signalAxes,'XLim');
            ud.signalAxesright=xlims(2);
            ud.signalAxesleft=xlims(1);
            ud.right3=round(0.9*(ud.signalAxesright-1))+1;
            ud.left3=round(0.1*(ud.signalAxesright-1))+1;
            set(ud.signalLine,'XData',1:ud.samps,'YData',sig);
            set(gcf,'Name',strcat('Oreja. Signal: - [',name,']'),...
                'WindowButtonMotionFcn','oreja motion');
            ud.name=name;
            ud.noplay=0;
            %             if length(name) >= 4
            %                 ud.noplay=strcmp(name(1:4),'myst');
            %             else
            %                 ud.noplay=0;
            %             end
            
            
            
            
            
            %cursors on annotation
            
            ud.annotLC = line('Parent',ud.annotAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.left4 ud.left4],'YData',get(ud.annotAxes,'Ylim'),...
                'ButtonDownFcn','oreja left');
            ud.annotRC = line('Parent',ud.annotAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.right4 ud.right4],'YData',get(ud.annotAxes,'Ylim'),...
                'ButtonDownFcn','oreja derecha');
            
            % cursors on channels
            ud.wavLC = line('Parent',ud.wavAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.left2 ud.left2],'YData',get(ud.wavAxes,'Ylim'),...
                'ButtonDownFcn','oreja left');
            ud.wavRC = line('Parent',ud.wavAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.right2 ud.right2],'YData',get(ud.wavAxes,'Ylim'),...
                'ButtonDownFcn','oreja derecha');
            
            %cursors on signal
            ud.signalLC = line('Parent',ud.signalAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.left3 ud.left3],'YData',get(ud.signalAxes,'Ylim'),...
                'ButtonDownFcn','oreja left');
            ud.signalRC = line('Parent',ud.signalAxes,'Color','black',...
                'EraseMode','xor','XData',[ud.right3 ud.right3],'YData',get(ud.signalAxes,'Ylim'),...
                'ButtonDownFcn','oreja derecha');
            
            
            
            drawnow
%             % find all annotations for given file
%             d=dir(fullfile(ud.annotdir,[name '.*']));
%             annotcount=0;
%             annotstrings=['hide transcription'];
%             for f=1:length(d)
%                 a=readAnnot(fullfile(ud.annotdir,d(f).name));
%                 if ~isempty(a.times)
%                     annotcount = annotcount+1;
%                     annots(annotcount)=a;
%                     [apath,aname,aext,aver] = fileparts(d(f).name);
%                     annotstrings=strvcat(annotstrings,aext(2:length(aext)));       
%                 end
%             end
   
            set([ud.saveAsopt ud.saveopt],'Enable','off');
            oreja 'replot';
            
            set(gcf,'KeyPressFcn','oreja keypressed');
            set(findobj(gcf,'Tag','SelectAll'),'Enable','on');
            set(findobj(gcf,'Tag','UnSelectAll'),'Enable','on');
            set(ud.loadopt,'Enable','on');   
            set(findobj('Tag','ZoomIn'),'CallBack','oreja ZoomIn');
            set(findobj('Tag','ZoomOut'),'CallBack','oreja ZoomOut');
            set(findobj('Tag','PlaySelect'),'Callback','oreja PlaySelect');
            set(findobj('Tag','PlayOriginal'),'Callback','oreja PlayOriginal');
            set(findobj('Tag','SelectAll'),'Callback','oreja SelectAll');
            set(findobj('Tag','UnSelectAll'),'Callback','oreja UnSelectAll');
            set(findobj(gcf,'Tag','mmenu'),'Enable','on');
            set(findobj(gcf,'Tag','PlayOriginal'),'Enable','on');
            set(findobj(gcf,'Tag','PlaySelect'),'Enable','on');
            set(findobj(gcf,'Tag','ZoomIn'),'Enable','on');
            set(findobj(gcf,'Tag','csignal'),'Enable','on');
            
%             if ud.changefilter == 0
%                 
%                 found = 0;
%                 counter = 0;
%                 flast = fopen(strcat(ud.path,'\oreja.ini'),'r');
%                 if flast ~= -1
%                     flastread=[];
%                     while ~feof(flast)
%                         filelast=fgets(flast);
%                         counter = counter + 1;
%                         flastread = [flastread filelast];
%                         if strcmp(filelast, fullpathname)
%                             found = 1;
%                         end
%                     end
%                     fclose(flast);
%                 end
%                 
%                 
%                 if found == 0
%                     if counter >= 4
%                         flast = fopen(strcat(ud.path,'\oreja.ini'),'w');
%                         if flast ~= -1
%                             fprintf(flast,'%s',flastread);
%                             fclose(flast);
%                         end
%                     end
%                     
%                     flast = fopen(strcat(ud.path,'\oreja.ini'),'a');
%                     if flast ~= -1
%                         fprintf(flast,'%s',fullpathname);
%                         fprintf(flast,'\n');
%                         uimenu('Parent', findobj(gcf,'Tag','signalmenu'), 'Callback', 'oreja loadlast', 'Tag', fullpathname, 'Label', strcat('...',fullpathname(max(1,length(fullpathname)-18):length(fullpathname))));
%                         fclose(flast);
%                     end
%                 end
%             end
%           
        end
            ud.changefilter=0;
            set(gcf,'UserData',ud);
        
        
    case 'makespect'
        ud=get(gcf,'UserData'); 
        set(ud.im,'CData',spectrogram(ud.sig+eps,ud.fs,10,1)');
        ca = caxis;
        disp(['ms:caxis=',num2str(ca)]);

        set(gcf,'UserData',ud);
        
        
    case 'newsig'
        ud=get(gcf,'UserData');
        set([ud.chansmenu] ,'Enable','on');
        axes(ud.spectAxes);
        ud.sig=ud.sig';
        % ud.sig=normalise(ud.sig(:)');
        get(ud.spectAxes,'XLim');
        
        sg=spectrogram(ud.sig+eps,ud.fs,10,1);
        sg=sg';
        size(ud.sig);
        [nc,nf]=size(sg);
        
        ud.spectAxesleft=1;
        ud.spectAxesright=nf;
        set(ud.sliderbar,'Max',nf);
        set(ud.sliderbar,'Min',0);
        set(ud.sliderbar,'Value',round(nf/2));
        
        
        ud.left1 = round(0.1*ud.spectAxesright);    % cursors
        ud.right1 = round(0.9*ud.spectAxesright);
        set(ud.spectAxes,'XLim',[1 nf],'YLim',[1 nc]);
        ud.im=imagesc(sg);
        set(ud.im,'ButtonDownFcn','','EraseMode','xor');

        set(ud.spectAxes,'XLim',[1 nf],'YLim',[1 nc]);
        drawnow;
        set(ud.spectstatus,'String','Click on spect to select the part of the signal you want to zoom ');
      
        set(gcf,'UserData',ud);
        
        oreja 'filter';
        oreja 'distort';

        
        
    case 'filter'
        ud=get(gcf,'UserData');
        set(ud.im,'ButtonDownFcn',''); 
        noise=rand(size(ud.sig));
        ud.samps=length(ud.sig);
        ud.sampsPer10ms=round(ud.fs/100);
        e=sampleEnv(ud.sig,ud.sampsPer10ms);
        le=length(e);
        ud.highcf=round(ud.fs/2.5);
        ud.cfs=MakeErbCFs(ud.lowcf,ud.highcf,ud.numchans);
        if (get(ud.boxfilter,'Value'))
            % Filter bandwidths to match cochlea (ERB)
            ud.bws=erb(ud.cfs);
        else
            % or.. Filter bandwidths to cover whole spectrum
            ud.bws=zeros(1,ud.numchans);
            % BW of first filter is special case: choose upper edge as
            % geometric mean of 1st two center frqs
            ud.bws(1) = 2*(sqrt(ud.cfs(2)*ud.cfs(1))-ud.cfs(1));  
            for c = 2:ud.numchans
                % BW of chan n chosen to just touch upper edge of band n-1
                ud.bws(c) = 2*(ud.cfs(c) - (ud.cfs(c-1)+0.5*ud.bws(c-1)));
            end
            % But don't let any BWs exceed 150% of CF
            ud.bws = min(1.5*ud.cfs, ud.bws);
            % Actual CFs passed to gammatoneEnvCarrier are scaled down to
            % account for broad rolloff tails?
            ud.bws = 0.6*ud.bws;
        end
        ud.envs=zeros(ud.numchans,ud.samps);
        ud.cars=zeros(ud.numchans,ud.samps);
        ud.noisecars=zeros(ud.numchans,ud.samps);
        ud.distorted=zeros(ud.numchans,ud.samps);
        ud.t=round(1:ud.sampsPer10ms:ud.samps);
        ud.t=ud.t(1:length(e));
        ud.le=le;
        %set(ud.wavAxes,'XLim',[1 le]) 
        set(ud.wavAxes,'YLim',[0 ud.numchans+1],'YTick',1:ud.numchans);
        % set(ud.chanAxes,'XLim',[1 3]);
        xlims=get(ud.chanAxes,'XLim');
        set(ud.chanAxes,'XLim',[1 3], 'YLim',[0 ud.numchans+1]);
        axes(ud.wavAxes);	
        v=[];
        for c=1:ud.numchans   
            axes(ud.wavAxes);		
            ud.lines(c)=CreateLine(c,le,ud.wavAxes);
            v=strvcat(v,num2str(round(ud.cfs(c))));
            axes(ud.chanAxes);	
            ud.squares(c)=CreateSquare(c,le,ud.chanAxes);
            
        end
        drawnow
        axes(ud.wavAxes);		
        
        set(ud.wavAxes,'YTickLabel',v);
        ylabel('channel cf');

        drawnow
        
        for c=1:ud.numchans
            [env,carrier]=gammatoneEnvCarrier(ud.sig,ud.fs,ud.cfs(c),ud.bws(c));
            [nenv,ncarrier]=gammatoneEnvCarrier(noise,ud.fs,ud.cfs(c),ud.bws(c));
            ud.envs(c,:) = env;
            ud.cars(c,:) = carrier;
            ud.noisecars(c,:) = ncarrier;
            ud.distorted(c,:) = env.*carrier;
            e=sampleEnv(ud.distorted(c,:),ud.sampsPer10ms);
            ChangeLine(ud.lines(c),c,e);
            set(ud.wavstatus,'String',['Computing Hilbert envelope for chan ' num2str(c)]); 
            
            drawnow;
        end
        set(ud.wavstatus,'String','Click to select/unselect channels');
        tmp=get(ud.wavAxes,'Xlim');
        ud.wavAxesleft=1;
        ud.wavAxesright=le;        
        set(gcf,'UserData',ud);

        oreja 'showSelected';
        oreja 'drawspect';
        
        
        
    case 'SelectAll'
        ud=get(gcf,'UserData');
        set(findobj(gcf,'Tag','mmenu'),'Enable','on');
        for chan=1:ud.numchans
            ud.selected(chan)=1;
            LowlightLine(ud.lines(chan));
            LowlightLine(ud.squares(chan));
        end
        set(gcf,'UserData',ud);
        oreja 'drawspect';
        oreja 'showSelected';
        
    case 'UnSelectAll'
        ud=get(gcf,'UserData');
        set(findobj(gcf,'Tag','mmenu'),'Enable','off');
        for chan=1:ud.numchans
            ud.selected(chan)=0;
            HighlightLine(ud.lines(chan));
            HighlightLine(ud.squares(chan));
        end
        set(gcf,'UserData',ud);
        oreja 'drawspect';
        oreja 'showSelected';
        
        
    case 'PlaySelect'
        ud=get(gcf,'UserData');
        wavplay(ud.selected*ud.distorted,ud.fs);
        
    case 'PlayOriginal'
        ud=get(gcf,'UserData');
        wavplay(ud.sig,ud.fs);
        
    case 'selectSignal'
        lud=get(gcbo,'UserData');
        chan=lud.chan;	
        ud=get(gcf,'UserData');
        if strcmp(get(gcf,'SelectionType'),'extend')
            wavplay(ud.distorted(chan,:),ud.fs);
        else
            if ~ud.selected(chan)
                LowlightLine(ud.lines(chan));
                LowlightLine(ud.squares(chan));
                ud.selected(chan)=1;
            else
                HighlightLine(ud.lines(chan));
                HighlightLine(ud.squares(chan));
                ud.selected(chan)=0;  
            end
            set(gcf,'UserData',ud);
            %refresh
            oreja 'drawspect';
            oreja 'replot';
        end
        
        if ud.selected*ones(length(ud.selected),1) > 0
            set(findobj(gcf,'Tag','mmenu'),'Enable','on');
        else
            set(findobj(gcf,'Tag','mmenu'),'Enable','off');
        end
        
    case 'drawspect'
        
        ud=get(gcf,'UserData');
        axes(ud.spectAxes);
        delete(get(ud.spectAxes,'Children'));
        sg=spectrogram((ud.selected*ud.distorted+eps),ud.fs,10,1);
        sg=sg';
        ud.im=imagesc(sg);
        ca = caxis;
        dBrange = 80;
        caxis([ca(2)-dBrange ca(2)]);
        % disp(['ds:caxis=',num2str(ca)]);

        ud.spectLC = line('Parent',ud.spectAxes,'Color','black',...
            'EraseMode','xor','XData',[ud.left1 ud.left1],'YData',get(ud.spectAxes,'Ylim'),...
            'ButtonDownFcn','oreja left');
        ud.spectRC = line('Parent',ud.spectAxes,'Color','black',...
            'EraseMode','xor','XData',[ud.right1 ud.right1],'YData',get(ud.spectAxes,'Ylim'),...
            'ButtonDownFcn','oreja derecha');
        
        set(gcf,'UserData',ud);
        
        
    case 'showSelected'
        ud=get(gcf,'UserData');
        for c=1:ud.numchans
            if ~ud.selected(c)
                HighlightLine(ud.lines(c));
                HighlightLine(ud.squares(c));
            else 
                LowlightLine(ud.lines(c));
                LowlightLine(ud.squares(c));
            end
        end
        %set(ud.lines(1),'YData',sampleEnv(ud.selected*ud.distorted,ud.sampsPer10ms));
        set(gcf,'UserData',ud);
        
    case 'changeNumchans'
        ud=get(gcf,'UserData');
        ud.numchans=getuicontrolvalue(ud.chansmenu,'int');
        
        ud.selected=ones(1,ud.numchans);
        delete(get(ud.wavAxes,'Children'));
        delete(get(ud.chanAxes,'Children'));
        set(gcf,'UserData',ud);
        set(findobj(gcf,'Tag','mmenu'),'Enable','off');
        oreja 'filter';
        
        ud=get(gcf,'UserData');
        ud.right1=round(ud.right2*((ud.spectAxesright-1)/(ud.wavAxesright-1)))+1;
        ud.right3=round(ud.right2*((ud.signalAxesright-1)/(ud.wavAxesright-1)))+1;
        ud.left1=round(ud.left2*((ud.spectAxesright-1)/(ud.wavAxesright-1)))+1;
        ud.left3=round(ud.left2*((ud.signalAxesright-1)/(ud.wavAxesright-1)))+1;
        
        ud.wavLC = line('Parent',ud.wavAxes,'Color','black',...
            'EraseMode','xor','XData',[ud.left2 ud.left2],'YData',get(ud.wavAxes,'Ylim'),...
            'ButtonDownFcn','oreja left');
        ud.wavRC = line('Parent',ud.wavAxes,'Color','black',...
            'EraseMode','xor','XData',[ud.right2 ud.right2],'YData',get(ud.wavAxes,'Ylim'),...
            'ButtonDownFcn','oreja derecha');
        
        set(ud.annotAxes,'Xlim',[ud.annotAxesleft ud.annotAxesright]);
        
        set(gcf,'UserData',ud);
        
        oreja 'plotcursors';
        %end
        
        
        %SLIDER
    case 'moveon'
        ud=get(gcf,'UserData');
        value_slider=get(ud.sliderbar,'Value');    
        axes(ud.spectAxes);
        nfv=get(ud.spectAxes,'XLim');
        xcenter=round((nfv(2)+nfv(1))/2);
        nfv(1)=nfv(1)+(value_slider-xcenter);
        nfv(2)=nfv(2)+(value_slider-xcenter);
        set(ud.spectAxes,'XLim',nfv);
        nf1=nfv(1);
        nf2=nfv(2);
        nfv2=get(ud.wavAxes,'XLim');
        ratio=(nfv2(2)-nfv2(1))/(nf2-nf1);
        
        xmax=nfv2(2)+(value_slider-xcenter)*ratio;
        xmin=nfv2(1)+(value_slider-xcenter)*ratio;
        set(ud.wavAxes,'XLim',[xmin xmax]);
        
        
        
        
    case 'distort'
        ud=get(gcf,'UserData');
        set(gcf,'UserData',ud');
        oreja 'drawspect'
        oreja 'redraw'
        
        
        
    case 'redraw'
        ud=get(gcf,'UserData');
        for c=1:ud.numchans
            ChangeLine(ud.lines(c),c,sampleEnv(ud.distorted(c,:),ud.sampsPer10ms));
        end
        
        
        
        %%------ ANNOTATIONS --------------
        
    case 'segmentID'
        ud=get(gcf,'UserData');
        
        label=getuicontrolvalue(ud.IDbox,'String');
        %         set(findobj(gcf,'Tag','thelabel'),'String',label);
        %         lud=get(ud.currentlabel,'UserData')
        set(ud.currentLabel,'String',label);
        
        
    case 'annotate'
        ud=get(gcf,'UserData');
        axes(ud.annotAxes);
        currentPoint=get(gca,'CurrentPoint');
        addAnnot(currentPoint(1));
        if ~isempty(ud.savename)
            set(ud.saveopt,'Enable','on');
        else
            set(ud.saveAsopt,'Enable','on');
        end
        set(gcf,'UserData',ud);
        
        
    case 'keypressed'
        % if delete key is pressed, delete any selected object
        double(get(gcf,'CurrentCharacter'));
        if double(get(gcf,'CurrentCharacter')) == 127
            f=findobj(gcf,'Selected','on');
            fud=get(f,'UserData');
            delete([fud.label f]);
            ud=get(gcf,'UserData');
            if ~isempty(ud.savename)
                set(ud.saveopt,'Enable','on');
            end  
            set(gcf,'UserData',ud);
        end  
        
        
        
    case 'select'
        ud=get(gcf,'UserData');  
        set(findobj(gcf,'Selected','on'),'Selected','off');   
        lud=get(gcbo,'UserData');
        %set([lud.line lud.label],'Selected','on');
        set([lud.line],'Selected','on');
        ud.currentLine=gcbo;
        ud.currentLabel=lud.label;
        set(gcf,'WindowButtonMotionFcn','oreja moveannot');
        set(gcf,'WindowButtonUpFcn','oreja releaseannot');
        set(gcf,'UserData',ud);
        
        
    case 'moveannot'
        ud=get(gcf,'UserData');  
        currentPoint=get(gca,'CurrentPoint');
        cp=currentPoint(1);
        lud=get(ud.currentLine,'UserData');
        set(lud.line,'XData',[cp cp]);
        xlim=get(gca,'XLim');
        set(lud.label,'Position',[cp+xlim(2)/100 0.1]);
        set(gcf,'UserData',ud);
        
    case 'releaseannot'
        set(gcf,'WindowButtonMotionFcn','oreja motion');
        ud=get(gcf,'UserData');
        if ~isempty(ud.savename)
            set(ud.saveopt,'Enable','on');
        end  
        set(gcf,'UserData',ud);
        
    case 'saveAsannot'
        ud=get(gcf,'UserData');
        
     
        savedir=pwd;

        cd(ud.annotdir);   
        [f,p]=uiputfile(fullfile(ud.annotdir,strcat(ud.name,'.annot')),'SAVE transcription as');
        cd(savedir);
        if f
            fid=fopen(fullfile(p,f),'w');
            if fid
                fclose(fid);
                ud.savename=fullfile(p,f);
                
                set(gcf,'UserData',ud);
                   cd(ud.path);
                oreja 'saveannot';
            else
                warndlg(['Cannot open file ' ud.savename ' for writing']);
                   cd(ud.path);
                oreja 'saveAsannot';
            end  
        end
     
        
    case 'saveannot'
        ud=get(gcf,'UserData');
        lins=findobj(ud.annotAxes,'Tag','Annot','LineWidth',0.5);
        x=[]; y=[];
        for i=1:length(lins)
            lud=get(lins(i),'UserData');
            xval=get(lud.line,'XData');   
            lab=get(lud.label,'String');   
            x=[x xval(1)];
            y=strvcat(y,lab);
        end
        x=round(x);
        [a,b]=sort(x);
        fid=fopen(ud.savename,'w');
        length(lins);
        if fid
            for i=1:length(lins)
                fprintf(fid,'%d\n%s\n',a(i),y(b(i),:));
            end
        else
            warndlg(['oreja: Problem writing to annotation file ' ud.savename]);
        end  
        set(ud.saveopt,'Enable','off');
        set(gcf,'UserData',ud);
        
    case 'loadannot'
        ud=get(gcf,'UserData');
    
        if checkSave                  % check if any transcription needs saving & save it    
            savedir=pwd;
            cd(ud.annotdir);
            [f,p]=uigetfile(fullfile(ud.annotdir,'*.annot'),'LOAD transcription file');
            cd(savedir);
            
            if f ~= 0

           
                
                annot=readAnnot(fullfile(p,f));
                delete(findobj(ud.annotAxes,'Tag','Annot'));
                delete(findobj(ud.annotAxes,'Tag','Annot2'));
                
                
                axes(ud.annotAxes);
                
                for L=1:length(annot.times)
                    addAnnot(annot.times(L),annot.labels(L,:));
                end
                ud.savename=fullfile(p,f);
                set(ud.saveopt,'Enable','off');
                set(ud.saveAsopt,'Enable','on');
                set(gcf,'UserData',ud);
            end
        end
        
        
        
        
        
        %TIME AND FREQUENCY INDEX    
        
    case 'left'
        switch (get(gcf,'SelectionType'))
            case 'normal'  
                set(gcf,'WindowButtonMotionFcn','oreja moveleft');
                set(gcf,'WindowButtonUpFcn','oreja release');  
            case 'open'
                oreja 'annotate';
        end   
        
    case 'derecha'
    
        switch (get(gcf,'SelectionType'))
            case 'normal'  
                set(gcf,'WindowButtonMotionFcn','oreja moveright');
                set(gcf,'WindowButtonUpFcn','oreja release');  
            case 'open'
                oreja 'annotate';
        end 
        
    case 'release'
        set(gcf,'WindowButtonMotionFcn','oreja motion');
        
    case 'moveright'
        ud=get(gcf,'UserData');
        cp=get(gca,'CurrentPoint');
        
        if (gca == findobj('Tag','WavWin'))
            xlims=get(ud.wavAxes,'XLim');
            ud.right2=min(max(cp(1),ud.left2),xlims(2)-2);
            ud.right1=round(ud.right2*((ud.spectAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.right3=round(ud.right2*((ud.signalAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.right4=round(ud.right2*((ud.annotAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.right2=round(ud.right2)+1;
        end
        
        if (gca == findobj('Tag','SpeechWin'))
                        xlims=get(ud.spectAxes,'XLim');
            ud.right1=min(max(cp(1),ud.left1),xlims(2)-2); 
            ud.right2=round(ud.right1*((ud.wavAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.right3=round(ud.right1*((ud.signalAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.right4=round(ud.right1*((ud.annotAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.right1=round(ud.right1)+1;
        end
        
        if (gca == findobj('Tag','SignalWin'))
            xlims=get(ud.signalAxes,'XLim');
            ud.right3=min(max(cp(1),ud.left3),xlims(2)-2);
            ud.right1=round(ud.right3*((ud.spectAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.right2=round(ud.right3*((ud.wavAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.right4=round(ud.right3*((ud.annotAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.right3=round(ud.right3)+1;
        end
        
        if (gca == findobj('Tag','AnnotWin'))
            xlims=get(ud.annotAxes,'XLim');
            ud.right4=min(max(cp(1),ud.left4),xlims(2)-2);
            ud.right1=round(ud.right4*((ud.spectAxesright-1)/(ud.annotAxesright-1)))+1;
            ud.right2=round(ud.right4*((ud.wavAxesright-1)/(ud.annotAxesright-1)))+1;
            ud.right3=round(ud.right4*((ud.signalAxesright-1)/(ud.annotAxesright-1)))+1;
            ud.right4=round(ud.right4)+1;
        end
        
        set(gcf,'UserData',ud);
        oreja plotCursors;
        
    case 'moveleft'
        ud=get(gcf,'UserData');
        cp=get(gca,'CurrentPoint');
        cp(1)=max(cp(1),1);
        if (gca == findobj('Tag','WavWin'))
            ud.left2=min(cp(1),ud.right2); 
            ud.left1=round(ud.left2*((ud.spectAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.left3=round(ud.left2*((ud.signalAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.left4=round(ud.left2*((ud.annotAxesright-1)/(ud.wavAxesright-1)))+1;
            ud.left2=round(ud.left2)+1;
        end
        
        if (gca == findobj('Tag','SpeechWin'))
            ud.left1=min(cp(1),ud.right1); 
            ud.left2=round(ud.left1*((ud.wavAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.left3=round(ud.left1*((ud.signalAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.left4=round(ud.left1*((ud.annotAxesright-1)/(ud.spectAxesright-1)))+1;
            ud.left1=round(ud.left1)+1;
        end
        
        if (gca == findobj('Tag','SignalWin'))
            ud.left3=min(cp(1),ud.right3);
            ud.left1=round(ud.left3*((ud.spectAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.left2=round(ud.left3*((ud.wavAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.left4=round(ud.left3*((ud.annotAxesright-1)/(ud.signalAxesright-1)))+1;
            ud.left3=round(ud.left3)+1;
        end
        if (gca == findobj('Tag','AnnotWin'))
            ud.left4=min(cp(1),ud.right4); 
            ud.left2=round(ud.left4*((ud.wavAxesright-1)/(ud.annotAxesright-1)))+1;
            ud.left3=round(ud.left4*((ud.signalAxesright-1)/(ud.annotAxesright-1)))+1;
            ud.left1=round(ud.left4*((ud.spectAxesright-1)/(ud.annotAxesright-1)))+1;
            
            ud.left4=round(ud.left4)+1; 
        end
        
     %   set(ud.timebox,'String',num2str(round((1000*ud.left3)/ud.fs)));
        set(gcf,'UserData',ud);
        oreja plotCursors;
        
        
        
    case 'motion'
        ud=get(gcf,'UserData');  
        cp=get(ud.spectAxes,'CurrentPoint')';
        cpp=get(ud.annotAxes,'CurrentPoint')';
        t=cp(1); f=cp(2);
        f2=cpp(2);
        %t=cpp(1); f=cpp(2);
        xlims=get(ud.spectAxes,'XLim');
        ylims=get(ud.spectAxes,'YLim');
        yannotlims=get(ud.annotAxes,'YLim');
        timespan=ud.samps/ud.fs;
        
        if isfield(ud,'signalAxesright')
            if and(t>xlims(1),t<xlims(2))
                
                if f2>yannotlims(2)
                    set(ud.timebox,'String',num2str(round((1000*(t-1)*(((ud.signalAxesright-1)/(ud.spectAxesright-1)))/ud.fs))));
                else
                    set(ud.timebox,'String','0');
                end
            else
                set(ud.timebox,'String','0');
            end
            
        end
        if and(f>ylims(1),f<ylims(2))
            if and(t>xlims(1),t<xlims(2))
                set(ud.freqbox,'String',num2str(round((f-1)*ud.fs/(2*(ylims(2)-ylims(1))))));
            end
            else
                set(ud.freqbox,'String','0');
            end
        
    case 'replot'
        %         ud=get(gcf,'UserData');
        %         set(ud.spectAxes,'XLim',[ud.spectAxesleft ud.spectAxesright]);
        %         set(ud.wavAxes,'XLim',[ud.wavAxesleft ud.wavAxesright]);
        %         set(ud.annotAxes,'Xlim',[ud.annotAxesleft ud.annotAxesright]);
        %         set(ud.signalAxes,'XLim',[ud.signalAxesleft ud.signalAxesright]);
        %         
        %         set(gcf,'WindowButtonMotionFcn','oreja motion');
        %         
        %         
        %         oreja plotCursors
        %  
        
        
        
    case 'goleft'
        % scroll leftwards
        ud=get(gcf,'UserData');
        shift=min(round(0.4*(ud.spectAxesright-ud.spectAxesleft)),ud.spectAxesleft);
        ud.spectAxesleft=round(ud.spectAxesleft-shift);
        ud.left1=round(ud.left1-shift);  
        ud.spectAxesright=round(ud.spectAxesright-shift);
        ud.right1=round(ud.right1-shift);
        if ud.spectAxesleft == 1
            set(ud.left1button,'Enable','off');
        end  
        set(gcf,'UserData',ud);
        oreja replot;
        
    case 'goright'
        % scroll rightwards
        ud=get(gcf,'UserData');
        shift=round(0.4*(ud.spectAxesright-ud.spectAxesleft));
        if ud.spectAxesright+shift > ud.samps
            shift = ud.samps-ud.spectAxesright;
        end  
        ud.spectAxesright=round(ud.spectAxesright+shift);
        ud.right1=round(ud.right1+shift);
        ud.spectAxesleft=round(ud.spectAxesleft+shift);
        ud.left1=round(ud.left1+shift);
        set(gcf,'UserData',ud);
        oreja replot  
        
    case 'plotCursors'
        ud=get(gcf,'UserData'); 
        set(ud.annotRC, 'XData',[ud.right1 ud.right1]);
        set(ud.spectRC, 'XData',[ud.right1 ud.right1]);
        set(ud.wavRC,'XData',[ud.right2 ud.right2]);
        set(ud.signalRC,'XData',[ud.right3 ud.right3]);
        set(ud.annotRC,'XData',[ud.right4 ud.right4]);
        set(ud.spectLC, 'XData',[ud.left1 ud.left1]);
        set(ud.wavLC,'XData',[ud.left2 ud.left2]);
        set(ud.signalLC,'XData',[ud.left3 ud.left3]);
        set(ud.annotLC,'XData',[ud.left4 ud.left4]);
        
        
        drawnow
        
    case 'play'
        ud=get(gcf,'UserData');
        ud.noplay;
        isempty(ud.sig);
        if and(~ud.noplay,~isempty(ud.sig))
            cp=get(ud.signalAxes,'CurrentPoint');
            cp=cp(1);
            if and(cp>=1,cp<=length(ud.sig))
                sound(ud.sig(ud.left3:ud.right3),ud.fs);
            else   
                sound(ud.sig,ud.fs);
            end
        end   
        
    case 'playBetweenLabels'
        ud=get(gcf,'UserData');
        if ~ud.noplay
            cp=get(ud.annotAxes,'CurrentPoint');
            cp=cp(1);  
            lins=findobj(ud.annotAxes,'Tag','Annot','LineWidth',0.5);
            x=[];
            for i=1:length(lins)
                lud=get(lins(i),'UserData');
                xval=get(lud.line,'XData');   
                x=[x xval(1)];
            end
            x=sort(round(x));
            leftlab=x(x<cp);
            if isempty(leftlab)
                leftlab=1;
            else
                leftlab=fliplr(leftlab); 
                leftlab=leftlab(1);
            end
            rightlab=x(x>cp);
            if isempty(rightlab)
                rightlab=ud.samps;
            else
                rightlab=rightlab(1);
            end
            sound(ud.sig(leftlab:rightlab),ud.fs);  
        end
        
        
        
    case 'ZoomIn'
        ud=get(gcf,'UserData');
        xcenter=round((ud.right1+ud.left1)/2);
        axes(ud.spectAxes);
        
        nfv=get(ud.spectAxes,'XLim');
        [limy limx]=size(get(ud.im,'CData'));
        
        nf1=nfv(1);
        nf2=nfv(2);
        if (xcenter-(nf2-nf1)*(1/4) < nf1)
            xmin=nf1;
            xmax=xcenter+(nf2-nf1)*(1/2);
        else if (xcenter+(nf2-nf1)*(1/4) > nf2)
                xmax=nf2;
                xmin=nf2-(nf2-nf1)*(1/2);
            else
                xmax=xcenter+(nf2-nf1)*(1/4);
                xmin=xcenter-(nf2-nf1)*(1/4);
            end
        end
        
        set(ud.spectAxes,'XLim',[xmin xmax]);
        
        set(ud.sliderbar,'Min',round((xmax-xmin)/2));
        set(ud.sliderbar,'Max',round(limx-(xmax-xmin)/2));
        
        set(ud.sliderbar,'Visible','on');
        tmp=get(ud.sliderbar,'SliderStep');
        
        set(ud.sliderbar,'SliderStep',tmp/2);
        
        nfv2=get(ud.wavAxes,'XLim');
        ratio=(nfv2(2)-nfv2(1))/(nf2-nf1);
        
        xmax=nfv2(2)+(xmax-nf2)*ratio;
        xmin=nfv2(1)+(xmin-nf1)*ratio;
        set(ud.wavAxes,'XLim',[xmin xmax]);
        %set(ud.sliderbar,'Value',round((xmin+xmax)/2));
        if get(findobj('Tag','ZoomOut'),'Enable'); 
            set(findobj('Tag','ZoomOut'),'Enable','On'); 
        end
        
        drawnow;
        
    case 'ZoomOut'
        ud=get(gcf,'UserData');
        xcenter=round((ud.right1+ud.left1)/2);
        axes(ud.spectAxes);
        nfv=get(ud.spectAxes,'XLim');
        nf1=nfv(1);
        nf2=nfv(2);
        [limy limx]=size(get(ud.im,'CData'));
        
        if (xcenter - (nf2-nf1)) < 1
            xmin=1;
            if xmin + (nf2-nf1)*2 > limx
                xmax=limx;
            else
                xmax=xmin+(nf2-nf1)*2;
            end
        else if (xcenter + (nf2-nf1)) > limx
                xmax=limx;
                if xmax - (nf2-nf1)*2 < 1
                    xmin=1;
                else
                    xmin=xmax-(nf2-nf1)*2;
                end
            else 
                xmax = xcenter + (nf2-nf1);
                xmin = xcenter - (nf2-nf1);
            end
        end
        set(ud.spectAxes,'XLim',[xmin xmax]);
        set(ud.sliderbar,'Min',round((xmax-xmin)/2));
        set(ud.sliderbar,'Max',round(limx-(xmax-xmin)/2));
        
        
        
        nfv2=get(ud.wavAxes,'XLim');
        ratio=(nfv2(2)-nfv2(1))/(nf2-nf1);
        
        xmax=nfv2(2)+(xmax-nf2)*ratio;
        xmin=nfv2(1)+(xmin-nf1)*ratio;
        set(ud.wavAxes,'XLim',[xmin xmax]);
        
        if and(xmin > 1,xmax <limx)
            tmp=get(ud.sliderbar,'SliderStep');
            set(ud.sliderbar,'SliderStep',tmp*2);
            
        else
            set(findobj('Tag','ZoomOut'),'Enable','Off'); 
            set(ud.sliderbar,'Visible','off');
        end
        
        drawnow;
        
    case 'quit'
        if strcmp(questdlg('Are you sure you want to close Oreja?'),'Yes');
            
            delete(gcf);
        end
         
    otherwise
        orejauievent('oreja',action);
        
        
end


function addAnnot(tim,label)

if nargin < 2
    label='?';
end  
ud=get(gcf,'UserData');
xlim=get(gca,'XLim');
thelabel=text(tim+xlim(2)/100,0.1,label,'ButtonDownFcn','oreja select',...
    'Erasemode','xor','Tag','Annot2','Color',[0.1 0.6 0.1]);
theline=line([tim tim],ud.YLim,'EraseMode','xor',...
    'Color',[0.1 0.6 0.1],'LineWidth',0.5,...
    'ButtonDownFcn','oreja select',...
    'Tag','Annot');
lud.label=thelabel;
lud.line=theline;
%set(thelabel,'Tag','thelabel');
set(thelabel,'UserData',lud);
set(theline,'UserData',lud);

%===========================




%-----
function y=delay(x,t)
lx=length(x)-t;
y=[zeros(1,t)  x(1:lx)];

%--------
function [env,carrier]=gammatoneEnvCarrier(x,fshz,cfhz,bwhz)
% envelope and carrier form of complex gammatone

bwcorrection=1.019;
wcf=2*pi*cfhz;
tpt=(2*pi)/fshz;
if nargin < 4
    bw=erb(cfhz)*bwcorrection;
else
    bw = bwhz*bwcorrection;
end
    
a=exp(-bw*tpt);
kT=[0:length(x)-1]/fshz;
gain=((bw*tpt)^4)/6;                          % integral of imp resp

q=exp(-j*wcf*kT).*x;                          % shift down to d.c.
p=filter([1 0],[1 -4*a 6*a^2 -4*a^3 a^4],q);	% filter: part 1
u=filter([1 4*a 4*a^2 0],[1 0],p);            % filter: part 2
env=gain*abs(u);
carrier=cos(wcf*kT + angle(u));

%-----------
function e=sampleEnv(env,interval)
pts=round(length(env)/interval)-1;
e=max(abs(reshape(env(1:(interval*pts)),interval,pts)));

%------
function l=CreateLine(p,len,parent)
x=[1:len (len-1):-1:2];
y=p*ones(size(x));
pat=patch(x,y,[0.9 0.9 0.7]);
set(pat,'Erasemode','xor');
l=line(1:len,p*ones(1,len));
set(l,'Color',[0.9 0.9 0.7],'Erasemode','xor','ButtonDownFcn',...
    'oreja selectSignal');
%set(l,'Color',[0.5 0.2 0.7],'Erasemode','xor');

lud.patch=pat;
lud.chan=p;
set(l,'UserData',lud);

function l=CreateSquare(p,len,parent)
x=[1:len (len-1):-1:2];
y=p*ones(size(x));
pat=patch(x,y,[0.9 0.9 0.7]);
set(pat,'Erasemode','xor');

l=line(2:2,p*ones(1,1),'MarkerEdgeColor',[0.9 0.9 0.7],'MarkerFaceColor',[0.9 0.9 0.7],'Marker','o','MarkerSize',10);
set(l,'Erasemode','xor','ButtonDownFcn',...
    'oreja selectSignal');

lud.patch=pat;
lud.chan=p;

set(l,'UserData',lud);

function ChangeLine(l,c,env)
lud=get(l,'UserData');
lenv=length(env);
set(lud.patch,'YData',[c+3*env c-3*fliplr(env(2:lenv-1))]);

function HighlightLine(l)
lud=get(l,'UserData');
set(lud.patch,'FaceColor',[0.9 0.9 0.7]);
set(l,'Color',[0.9 0.9 0.7]);
set(l,'MarkerFaceColor',[0.9 0.9 0.7]);
set(l,'MarkerEdgeColor',[0.9 0.9 0.7]);


function LowlightLine(l)
lud=get(l,'UserData');
set(lud.patch,'FaceColor',[0.5 0.6 0.3]);
set(l,'Color',[0.5 0.7 0.3]);
set(l,'MarkerFaceColor',[0.5 0.7 0.3]);
set(l,'MarkerEdgeColor',[0.5 0.7 0.3]);





function ok=checkSave
% determine if user wants to save transcription first, or cancel, or continue
% return ok if anything but cancel
ok=1;
ud=get(gcf,'UserData');
if strcmp(get(ud.saveopt,'Enable'),'on')
    ans=questdlg('Save transcription?');
    if strcmp(ans,'Yes')
        if ud.savename
            oreja 'saveannot';
        else  
            oreja saveAsannot;
        end  
    elseif strcmp(ans,'Cancel')
        ok=0;
    end
end  


%===A utility to center the window on the screen============
function pos = centerfig(width,height)

% Find the screen size in pixels
screen_s = get(0,'ScreenSize');
pos = [screen_s(3)/2 - width/2, screen_s(4)/2 - height/2, width, height];

%--------COSAS QUE NO VALEN-----------
%case 'manipulation'
%  ud=get(gcf,'UserData');
%  ud.manipulation=getuicontrolvalue(ud.manipulationmenu);
%  set(gcf,'UserData',ud);
%  oreja 'distort'	

%case 'attenuation'
%  ud=get(gcf,'UserData');
%  ud.attenuation=getuicontrolvalue(ud.attenuationmenu);
% set(gcf,'UserData',ud);
% oreja 'distort'	

%case 'channeliminated'
% ud=get(gcf,'UserData');
%ud.distorttime=getuicontrolvalue(ud.timemenu,'int');
% set(gcf,'UserData',ud);
% oreja 'distort'	


function a=readAnnot(f)
% reads annotation from file f
% form assumed is list of st end label  or just st label
fid=fopen(f,'r');
a=emptyAnnot;
if fid
    [lims,read]=fscanf(fid,'%s',1);
    [sym,read]=fscanf(fid,'%s',1);
    lims=str2num(lims);
    
    
    while read
        a.times = [a.times lims(1)];
        a.labels= strvcat(a.labels,sym);
        [lims,read]=fscanf(fid,'%s',1);
        lims=str2num(lims);
        if read
            [sym,read]=fscanf(fid,'%s',1);
        end
    end
end
fclose(fid);

function a=emptyAnnot
a.times=[];
a.labels=[];

